//
//  SQLDriver.m
//  SQLDriver
//
//  Created by Stuart Ball on 3/4/2013.
//
//This example requires WeatherCat Version 1.2.0 build 535 or later.

#import "SQLDriver.h"
#import "SQLPrefsWindow.h"

//Private declarations
@interface SQLDriver (Private)
-(void) launchupdateThread;
@end

//Implementation
@implementation SQLDriver

// initWithWeatherCat is called just after our plugin has been loaded.
-(void) initWithWeatherCat:(AppPlugInManagerAPI*) controller
{
 NSMutableDictionary *appDefaults;	// dictionary of these application defaults
 NSUserDefaults *defaults;		// user defaults object used to store preferences

	myPort = [NSPort port];


//Set up our defaults
    defaults = [NSUserDefaults standardUserDefaults];
    appDefaults = [NSMutableDictionary dictionary];
    
    [appDefaults setObject:@"" forKey:@"com.trixology.SQLDriver.URL"];
    [appDefaults setObject:@"3306" forKey:@"com.trixology.SQLDriver.Port"];    
    [appDefaults setObject:@"60" forKey:@"com.trixology.SQLDriver.UploadPeriodInSecondsFast"];    
    [appDefaults setObject:@"1" forKey:@"com.trixology.SQLDriver.updateRateDivider"];    
    [appDefaults setObject:@"5" forKey:@"com.trixology.SQLDriver.historicalUpdatePeriod"];    
    [appDefaults setObject:@"0" forKey:@"com.trixology.SQLDriver.Enabled"];    
    [appDefaults setObject:@"WeatherCat" forKey:@"com.trixology.SQLDriver.DBName"];    
    [appDefaults setObject:@"" forKey:@"com.trixology.SQLDriver.UserName"];    
    
    [defaults registerDefaults:appDefaults];
	
    
    myWeatherCat=controller;	//Now we can call stuff declared in AppPlugInManagerAPI.h via myWeatherCat
    NSBundle* myBundle =[ NSBundle bundleForClass:[self class]];	//So we can get at localised strings
    [myWeatherCat logString:[NSString localizedStringWithFormat:NSLocalizedStringFromTableInBundle (@"SQLDriver init", @"Localizable",myBundle, @"")]]; //Display this in WeatherCat's log window

    [self launchupdateThread];  //the thread that does all the work
 
    [myWeatherCat logString:[NSString localizedStringWithFormat:NSLocalizedStringFromTableInBundle (@"SQLDriver initted", @"Localizable",myBundle, @"")]]; //Display this in WeatherCat's log window


}

//WeatherCat needs to know what version of the interface we support
//For now there's only one - '1'.
- (unsigned int)interfaceVersion
{
	return 1;   //Currently must return 1 (otherwise there's a good chance WeatherCat will not run you)
}

//Return our version as an unsigned integer
- (unsigned int) pluginVersion
{ 
	return 100;	//Return your version (as an integer - i.e. 100 is 1.0.0, 212 is 2.1.2 etc)
}

//Return our name - this appears in WeatherCat's 'Tools' menu.
-(NSString*) name
{
	
	return @"SQLDriver";
}

//Return our category
-(plugInCategory) category
{
	return general;	//return one of plugInCategory (see WeatherCatPlugInProtocol.h)
}

-(void) initWindow
{
	myWindow = [SQLPrefsWindow alloc];

//get the path to the right nib

	NSBundle* myBundle =[ NSBundle bundleForClass:[self class]];
	NSString* nibPath = [myBundle pathForResource: @"com.trixology.SQL.PrefsWindow" ofType:@"nib"];

// Initialize with the nib
	myWindow	= [myWindow initWithWindowNibPath: nibPath owner: myWindow];
	[myWindow setMyController:self APIManager:myWeatherCat];

}

//Called when user selects us from the tools menu 
-(void) selectedFromMenu
{
	if (myWindow==0)
	{
		[self initWindow];
	}

//Show our window
	[myWindow showWindow:self];
    [myUpdateThread doUpdatePrefs]; //make sure the window is updated with the current state
}

//This is not yet implemented within WeatherCat and will not be called. 
//it is designed to synchronise your plugin with WeatherCats sampling in
//the future.
-(void) weatherUpdate
{
	    //Not implemented within WeatherCat yet - currently this will never be called
}

//Called every 60 seconds (as set up in initWithWeatherCat) 
//We are on the main controlelr thread here so don't block
//Nothing to do here, we're all driven from our updateThread
-(void) update
{
}

-(void) generateString
{

}

-(void) launchupdateThread
{
    if (myPort)
    {
		myUpdateThread=[[SQLUpdateThread alloc]init];
		[myUpdateThread setup:self];    //intialise it
		[NSThread detachNewThreadSelector:@selector(updateThreadMethod:) toTarget:myUpdateThread withObject:myPort];
	}
}

//This is called when WeatherCat quits and gives us a chance to tidy up 
-(void) quit
{
    [myUpdateThread quit];
}


-(AppPlugInManagerAPI*) getWeatherCat
{
    return myWeatherCat;
}


-(void)updateWindowWithState:(int)state packets_sent:(int)sent packets_not_acked:(int)bad_count sent:(int)bytes_sent time_sent:(NSTimeInterval) SentTime av_time:(NSTimeInterval) av_time
{
    if (myWindow)
    {
        [myWindow updateWithState:state packets_sent:sent packets_not_acked:bad_count sent:bytes_sent time_sent:SentTime av_time: av_time];
    }
}

//Caleld from our window to set the run state (i.e. stop/start buttons)
-(void) SetRunState:(BOOL) run_state
{
    run_state_copy=run_state;   //cache it so we don't have to ask
    [myUpdateThread SetRunState:run_state];
}

-(int) getRunState
{
    return run_state_copy;
}

-(void) setCurrentState:(current_update_state) state
{
    [myUpdateThread setCurrentState: state];
}

-(current_update_state) getCurrentState
{
    return [myUpdateThread getCurrentState];
}

-(int) length
{
    return 0;
}

-(void) resetTimings
{
    [myUpdateThread resetTimings];
}

-(NSWindow*) getWindow
{
    return [myWindow window];
}

-(void) setUploadString:(NSString*) string
{
    [myWindow setMiscString:string];
}

-(void) setUploadProgressIndicator:(BOOL)display value:(double)value max_value:(double)max_value;
{
    [myWindow setUploadProgressIndicator:display value:value max_value:max_value];
}
@end
