//
//  SQLUpdateThread.m
//  SQLDriver
//
//  Created by Stuart Ball on 06/05/2013.
//  Copyright (c) 2013 Trixology. All rights reserved.
//

#import "SQLUpdateThread.h"
#import "SQLDriver.h"


@implementation SQLUpdateThread

-(void) setup:(SQLDriver *)value 
{
	myController=value;
	thread_stop=NO;
    myAPIManager=[myController getWeatherCat];
    current_state=updater_disabled;

    BOOL enabled=[[NSUserDefaults standardUserDefaults] boolForKey:@"com.trixology.SQLDriver.Enabled"];
    if (enabled)
    {
        current_state=updater_waiting_to_start;
        [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:0 av_time:0];
    }
}


- (void) updateThreadMethod:(id)anObject
{

    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init]; // If I make an Cocoa calls I don't want to leak...
	
	//Set up a timer to start us
    RunTimer=[NSTimer scheduledTimerWithTimeInterval:10 //Wait 10 seconds before trying to do anything
			target:self
            selector:@selector(doProcessing)
            userInfo:nil 
            repeats:NO];

	// Create a port for intra-thread communication
    // We actually don't use it here
	distantPort = (NSPort*)anObject;
	[[NSRunLoop currentRunLoop] run];		
    [pool release];	//do this or we'll leak
}

-(void) doProcessing
{  

    update_rate_in_secs=[[NSUserDefaults standardUserDefaults] integerForKey:@"com.trixology.SQLDriver.UploadPeriodInSecondsFast"];
    //Reset the timer
    //Set up a timer to call us next time
    if (RunTimer)
    {
        [RunTimer invalidate];
    }

    RunTimer=[NSTimer scheduledTimerWithTimeInterval:update_rate_in_secs
            target:self
    selector:@selector(doProcessing)
    userInfo:nil 
    repeats:NO];

    if ([myAPIManager isDataValid]==NO || (current_state==updater_disabled))    //wait until WeatherCat is ready and we can run
    {
        if (RunTimer)
        {
            [RunTimer invalidate];
        }

        RunTimer=[NSTimer scheduledTimerWithTimeInterval:5  //can't run now, try again in 5 seconds
                target:self
        selector:@selector(doProcessing)
        userInfo:nil 
        repeats:NO];
        return;
    }

    if (thread_stop==NO)	//we can run
    {
        if (upload_in_progress==YES)
        {
            //Try again later
            return;
        }


        NSString* URL_string=[[NSUserDefaults standardUserDefaults] stringForKey:@"com.trixology.SQLDriver.URL"];
        NSString* user_string=[[NSUserDefaults standardUserDefaults] stringForKey:@"com.trixology.SQLDriver.UserName"];
        NSString* password_string=[myAPIManager getPasswordForKey:@"SQLDriverPassword"];    //Requires WeatherCat 1.2.0

        
        if ([URL_string compare:@""]==NSOrderedSame)
        {
            current_state=updater_no_url_error;
        }
        if ([password_string compare:@""]==NSOrderedSame)
        {
            current_state=updater_no_password_error;
        }
        if ([user_string compare:@""]==NSOrderedSame)
        {
            current_state=updater_no_user_error;
        }
        
        //Decide if we can upload
        
        if ((current_state!=updater_no_url_error) && (current_state!=updater_no_password_error) && (current_state!=updater_no_user_error))
        {
            //Check if it's a new database - if so we need to create the tables then offer to populate with existing WeatherCat data
            if (checked_tables==NO) 
            {
                checked_tables=YES;
                if ([self checkIfNewDB])    //no tables in DB yet?
                {
                    [self createTables];    //Yes, create the tables
                    [self doInitialPopulate];   //Handle initial population from existing data
                    return; 
                }
            }
            
            if ([self checkForMissingData]) //Check if any data is missing from the DB and upload it if so
            {
                return; //return here just in-case the upload was cancelled with the stop button (so we don't upload live data and overwrite the last upload date)
            }
            if (slow_update_counter==0)
            {
                [self doUploadSlow];    //this also uploads the fast data
                slow_update_counter=[[NSUserDefaults standardUserDefaults] floatForKey:@"com.trixology.SQLDriver.updateRateDivider"];
            }
            else
            {
                [self doUploadFast];
            }
            slow_update_counter--;  //divider for slow data update
        }
        else
        {
            [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:0 av_time:AverageTime];

        }
    }        

//All done
return;

}

-(BOOL) checkForMissingData //returns Yes if it did upload
{
NSDate* lastUploadDate=[self getLastUploadDate];
NSDate* now=[NSDate date];

NSTimeInterval now_secs=[now timeIntervalSinceReferenceDate];
NSTimeInterval upload_secs=[lastUploadDate timeIntervalSinceReferenceDate];
NSTimeInterval difference=now_secs-upload_secs;
int upload_period_in_seconds=120;
//Check users upload rate in case it's greater than this
int user_upload_period_in_seconds=[[NSUserDefaults standardUserDefaults] integerForKey:@"com.trixology.SQLDriver.UploadPeriodInSecondsFast"];
int slow_update_rate=[[NSUserDefaults standardUserDefaults] floatForKey:@"com.trixology.SQLDriver.updateRateDivider"];

user_upload_period_in_seconds*=slow_update_rate;

if (user_upload_period_in_seconds>upload_period_in_seconds)
{
    upload_period_in_seconds=user_upload_period_in_seconds;
}

upload_period_in_seconds*=2;
if (difference>upload_period_in_seconds)    //if more than 2 minutes or four times the user slow upload rate, upload missing data
{

    //upload from lastUploadDate
    if ([self internetConnect])
    {
        NSBundle* myBundle =[ NSBundle bundleForClass:[self class]];	//So we can get at localised strings

        NSString* temp=[NSString localizedStringWithFormat:NSLocalizedStringFromTableInBundle (@"Uploading missing data to database.", @"Localizable",myBundle, @"")];
        [self sendLogMessage:temp];

        current_state=updater_disabled; //lets stop everything
        [self uploadFromDate:lastUploadDate];
        mysql_close(con);

        [self resetTimings];
        temp=[NSString localizedStringWithFormat:NSLocalizedStringFromTableInBundle (@"Finished uploading missing data to database.", @"Localizable",myBundle, @"")];
        [self sendLogMessage:temp];
        return YES;
    }

}
return NO;

}

//Tables have been created
-(void) doInitialPopulate
{
    if ([self internetConnect])
    {
        current_state=updater_disabled; //lets stop everything
        [self populateWithExistingData];
        mysql_close(con);
        [self resetTimings];

    }
    
}

-(void) populateWithExistingData
{
    NSBundle* myBundle =[ NSBundle bundleForClass:[self class]];	//So we can get at localised strings

    NSString* temp=[NSString localizedStringWithFormat:NSLocalizedStringFromTableInBundle (@"New database, uploading existing data.", @"Localizable",myBundle, @"")];
    [self sendLogMessage:temp];

    [self uploadFromDate:[[myController getWeatherCat] getDateOfOldestSample] ];

    temp=[NSString localizedStringWithFormat:NSLocalizedStringFromTableInBundle (@"Finished initial import to database.", @"Localizable",myBundle, @"")];
    [self sendLogMessage:temp];

}

-(void) uploadFromDate:(NSDate*) upload_date
{
//Because we are on our own thread, we can just sit here now, sending the data and everything else will carry on running
    NSTimeInterval sampleTime=[upload_date timeIntervalSinceReferenceDate] ;
    NSTimeInterval now=[[NSDate date] timeIntervalSinceReferenceDate];


//Now we can loop around, incrementing sampleTime until we're up to the current time, getting the data
//and inserting it into the DB
    int import_interval=[[NSUserDefaults standardUserDefaults] floatForKey:@"com.trixology.SQLDriver.historicalUpdatePeriod"];  //in minutes
    int number_of_samples_to_upload=((now-sampleTime)/60)/import_interval;    //How many samples we're going to be uploading
    int upload_count=0;
    int error_count=0;
    int start_calc_interval=1000;
    current_state=updater_populating;
    [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:0 av_time:0];
    [myController setUploadProgressIndicator:YES value:0 max_value:number_of_samples_to_upload];
    
    //Create a pool so we can ditch all the pointers whenever we update the progress bar (otherwise these dictionaries will just hang around until this loop is finished)
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    NSDate* sample_date=0;
    while((sampleTime<now) && (thread_stop==NO))
    {

        sample_date=[NSDate dateWithTimeIntervalSinceReferenceDate:sampleTime];
        float progress=((float)upload_count/(float)number_of_samples_to_upload)*100.0;
        NSDictionary* myData=[myAPIManager getWeatherDataAtDate:sample_date];  //Get all available data

        [self sendWeatherData:myData date:sample_date];
        [self sendWindDataHistoric:myData date:sample_date];
        [self sendExtraData:myData date:sample_date];
        if (internal_sql_error != MYSQL_SQL)    //No error?
        {
            upload_count++;
            sampleTime+=import_interval*60;
            error_count=0;
        }
        else    //error
        {
            error_count++;
            usleep(2000000);    //Wait 2 seconds before retrying
            if (error_count>30) //Try 30 times
            {
                //We have failed, log it and end.
                current_state=updater_disabled; //Stop 
                [self sendLogMessage:@"Failed to import initial data."];
                break;  //make sure we clean up the display
            }
        }
        
        if (upload_count%start_calc_interval==0)    //Update progress/percentage indication
        {
            int number_of_samples_remaining=number_of_samples_to_upload-upload_count;
            NSString* uploadString=uploadString=[NSString localizedStringWithFormat:@"Historical: %.1f%% (%iK samples remaining)", progress, number_of_samples_remaining/1000];
            [myController setUploadString:uploadString];
            [myController setUploadProgressIndicator:YES value:upload_count max_value:number_of_samples_to_upload];
            [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:0 av_time:0];
            
            now=[[NSDate date] timeIntervalSinceReferenceDate]; //Keep now updated so we don't lose any data during the upload
            
            current_state=updater_ok;
            
            [self setLastUploadDateWithDate:sample_date];  //Save our progress
            
            //Memory management
            [pool drain];   //release everything that was allocated...
            pool = [[NSAutoreleasePool alloc] init];    //...and get a new pool
        }
        
        
    }

    
    NSString* uploadString=[NSString localizedStringWithFormat:@""];
    [myController setUploadString:uploadString];
    [myController setUploadProgressIndicator:NO value:upload_count max_value:number_of_samples_to_upload];
    if (thread_stop==NO)    //If the user didn't stop us, update last time to 'now' and indicate all OK
    {
        current_state=updater_ok;
        [self setLastUploadDateWithDate:[NSDate date]];  //We're all done - mark now as the last upload time
    }
    [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:SentTime av_time:AverageTime];
    
    [pool drain];   //release everything that was allocated that we previously haven't released

}

+(int) count
{
    return 0;
}

-(BOOL) checkIfNewDB
{
BOOL return_val=YES;
    if ([self internetConnect])
    {
        MYSQL_RES* result=mysql_list_tables(con, NULL);
        
        my_ulonglong row_count=result->row_count;
        if (row_count!=0)
        {
            return_val=NO;  //we have some tables
        }
        mysql_free_result(result);   
        mysql_close(con);
    }
    else
    {
        return NO;
    }
return return_val;
}

-(void) doUploadFast
{
    SentTime=[[NSDate date] timeIntervalSince1970];
    [self sendDataFast];

	CumulativeFlightTime+=[[NSDate date] timeIntervalSince1970]-SentTime;
	AverageTime=CumulativeFlightTime/sent_packets;
    [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:SentTime av_time:AverageTime];
    return;
}



-(void) doUploadSlow
{
    SentTime=[[NSDate date] timeIntervalSince1970];
    [self sendDataSlow];

	CumulativeFlightTime+=[[NSDate date] timeIntervalSince1970]-SentTime;
	AverageTime=CumulativeFlightTime/sent_packets;
    [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:SentTime av_time:AverageTime];
    return;
}



-(void) SetRunState:(BOOL) run_state
{
	if (run_state==YES)   //start
	{
            [[NSUserDefaults standardUserDefaults] setObject:@"YES" forKey:@"com.trixology.SQLDriver.Enabled"];
            thread_stop=NO;
            current_state=updater_waiting_to_start; 
	}
	else
	{
		[[NSUserDefaults standardUserDefaults] setObject:@"NO" forKey:@"com.trixology.SQLDriver.Enabled"];
		current_state=updater_disabled; //stopped
        thread_stop=YES;
        [self resetTable];  //make sure we try to create the tables when started (in-case the user had been diddling with the DB while we were stopped)
	}
    [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:0 av_time:AverageTime];

}

-(void) setCurrentState:(current_update_state) state
{
    current_state=state;
}

-(current_update_state) getCurrentState
{
    return current_state;
}

-(void) doUpdatePrefs
{
    [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:SentTime av_time:AverageTime];

}


// ------------------------------------


-(void) createTables
{

    if ([self internetConnect])
    {
        [self createWindTable];
        [self createExtraTable];
        [self createDataTable];
        [self createPrivateTable];
        if (internal_sql_error==MYSQL_SQL)
        {
            [self SetRunState:NO];
        }
        mysql_close(con);
    }    
}

-(void)sendDataFast
{
        current_state=updater_transmitting;
        [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:0 av_time:AverageTime];

        if ([self internetConnect])
        {
            [self sendWindData]; //wind is updated
            if (internal_sql_error==MYSQL_SQL)
            {
                [self SetRunState:NO];
            }

            mysql_close(con);
        }
        else
        {
            //didn't connect
            [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:0 av_time:AverageTime];
            usleep(2000000);
        }
        
}


-(void)sendDataSlow
{
    current_state=updater_transmitting;
    [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:0 av_time:AverageTime];
    if ([self internetConnect])
    {
        NSDictionary* myData=[myAPIManager getCurrentWeatherData];  //Get all available data
        NSDate* now=[NSDate date];
        [self sendWeatherData:myData date:now];
        if (internal_sql_error==MYSQL_SQL)
        {
            [self SetRunState:NO];
            mysql_close(con);
            return;
        }
        [self sendWindData];
        if (internal_sql_error==MYSQL_SQL)
        {
            [self SetRunState:NO];
            mysql_close(con);
            return;
        }
        [self sendExtraData:myData date:now];
        if (internal_sql_error==MYSQL_SQL)
        {
            [self SetRunState:NO];
            mysql_close(con);
            return;
        }
        
        //If we get here, we have successfully updated all three tables
        [self setLastUploadDate];

        mysql_close(con);
    }
    [myController updateWindowWithState:current_state packets_sent:sent_packets packets_not_acked:ack_packets sent:bytes_sent time_sent:0 av_time:AverageTime];

}


#define BUF_SIZE 128

#define CLIENT_LONG_PASSWORD     	1
#define CLIENT_CONNECT_WITH_DB   	8
#define CLIENT_PROTOCOL_41       	512
#define CLIENT_INTERACTIVE         	1024
#define CLIENT_SECURE_CONNECTION 	32768
#define CLIENT_RESERVED         	16384


//Internet
- (BOOL)internetConnect 
{
    const char* server;   //[400];
    int port_number=[[NSUserDefaults standardUserDefaults] integerForKey:@"com.trixology.SQLDriver.Port"];

    NSString* serverURL=[[NSUserDefaults standardUserDefaults] objectForKey:@"com.trixology.SQLDriver.URL"];
    if ([serverURL length]<400)
    {
        server= [serverURL UTF8String];
    }

    if (server==0)
    {
        return NO;
    }

    con = mysql_init(NULL);

    if (con == NULL) 
    {
      fprintf(stderr, "%s\n", mysql_error(con));
      return NO;
    }

 NSString* user1=[[NSUserDefaults standardUserDefaults] stringForKey:@"com.trixology.SQLDriver.UserName"];
 
 NSString* password1=[myAPIManager getPasswordForKey:@"SQLDriverPassword"];

 NSString* db1=[[NSUserDefaults standardUserDefaults] stringForKey:@"com.trixology.SQLDriver.DBName"];

    int result=mysql_real_connect(con, [serverURL UTF8String], [user1 UTF8String], [password1 UTF8String], [db1 UTF8String], port_number, NULL, 0);
    if (result==0)  //didn't connect
    {
        unsigned int error=mysql_errno(con);
        NSString* logMessage=[NSString stringWithFormat:@"%s\n",mysql_error(con)];
        if (error==1045)
        {
            current_state=updater_pwd;
        }
        else
        if (error==1049)
        {
            current_state=updater_unknown_db;
        }
        else
        if (error==2003)
        {
            current_state=updater_cant_connect;
        }
        else
        {
            current_state=updater_sql_error;
        }
        NSLog(logMessage);
        [self sendLogMessage:logMessage];
        mysql_close(con);
        return NO;
    } 
     
    //Did connect to server
    current_state=updater_ok;

return YES;


}

-(int) sendWeatherData:(NSDictionary*) myData date:(NSDate*) dateOfData
{
NSString* send_string=@"INSERT INTO WeatherData ";
NSString* data=@"VALUES (";
NSString* columns=@"(";
NSString* temp;
int data_count=0;
//Add date and time
	data=[data stringByAppendingString:@"'"];
	NSTimeZone *timeZoneUTC = [NSTimeZone timeZoneWithName:@"UTC"];
	NSString* dateString=[dateOfData description];
	NSCalendarDate* tempCalDate=[NSCalendarDate dateWithString:dateString];
	[tempCalDate setTimeZone:timeZoneUTC];
	NSString* utcDateString=[tempCalDate descriptionWithCalendarFormat:@"%Y-%m-%d %H:%M:%S"];
	data=[data stringByAppendingString:utcDateString];
	data=[data stringByAppendingString:@"'"];
    columns=[columns stringByAppendingString:@"Time"];


//Ext temp
    
    float value=[[[myData objectForKey:@"external_temperature"]  objectForKey:@"value"] floatValue];
    BOOL valid=[[[myData objectForKey:@"external_temperature"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {    
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", Temperature_C"];
    }
//dewpoint
    value=[[[myData objectForKey:@"dewpoint"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"dewpoint"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", Dewpoint_C"];
    }



//Ext hum
    value=[[[myData objectForKey:@"external_rh"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"external_rh"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", Humidity"];
    }


//Pressure
    value=[[[myData objectForKey:@"pressure"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"pressure"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.2f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", Pressure_mB"];
    }

//Rain/h
    value=[[[myData objectForKey:@"rain_rate"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"rain_rate"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", RainRate_mm"];
    }

//Raw Rain

    value=[[[myData objectForKey:@"raw_rain"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"raw_rain"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", RawRain_mm"];
    }

//Daily Rain
/*
    value=[[[myData objectForKey:@"rain_daily"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"rain_daily"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", DailyRain_mm"];
    }

//Monthly Rain
    value=[[[myData objectForKey:@"rain_monthly"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"rain_monthly"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", MonthlyRain_mm"];
    }

//Yearly Rain
    value=[[[myData objectForKey:@"rain_yearly"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"rain_yearly"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", YearlyRain_mm"];
    }
*/

//Solar
    value=[[[myData objectForKey:@"solar_radiation"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"solar_radiation"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", Solar_wm2"];
    }

//UV
    value=[[[myData objectForKey:@"uv"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"uv"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", UV_index"];
    }

//Internal temp
    value=[[[myData objectForKey:@"internal_temperature"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"internal_temperature"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", TemperatureInt_C"];
    }

//Internal Humidity
    value=[[[myData objectForKey:@"internal_rh"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"internal_rh"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data_count++;
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", HumidityInt"];
    }


//Conditions
//    NSString* conds=[myAPIManager getCurrentConditionsString];
    NSString* conds=[[myData objectForKey:@"current_conditions"]  objectForKey:@"value"];
    data=[data stringByAppendingString:@",'"];
    data=[data stringByAppendingString:conds];
    data=[data stringByAppendingString:@"'"];
    columns=[columns stringByAppendingString:@", Conditions"];




	data=[data stringByAppendingString:@");"];
	columns=[columns stringByAppendingString:@") "];





    send_string=[send_string stringByAppendingString:columns];
    send_string=[send_string stringByAppendingString:data];

    if (data_count>0)   //anything to send?
    {

        int error=mysql_query(con, [send_string UTF8String]);

        if (error == MYSQL_OK)
        {
            internal_sql_error = MYSQL_OK;
            bytes_sent+=[send_string length];
        }
        else
        {
            unsigned int error=mysql_errno(con);
            NSString* logMessage=[NSString stringWithFormat:@"%s\n",mysql_error(con)];
            if (error==1045)
            {
                current_state=updater_pwd;
            }
            else
            if (error==1049)
            {
                current_state=updater_unknown_db;
            }
            else
            if (error==2003)
            {
                current_state=updater_cant_connect;
            }
            else
            {
                current_state=updater_sql_error;
            }
            NSLog(logMessage);
            [self sendLogMessage:logMessage];
            internal_sql_error = MYSQL_SQL;
            current_state=updater_unknown_db;
        }
    }
    else
    {
            internal_sql_error = MYSQL_OK;
    }
return 0;
}




-(int) sendExtraData:(NSDictionary*) myData date:(NSDate*) dateOfData
{
NSString* send_string=@"INSERT INTO ExtraData ";
NSString* data=@"VALUES (";
NSString* columns=@"(";
NSString* temp;
int data_count=0;   //count of items to send - we don't send if there's none.
//Add date and time
	data=[data stringByAppendingString:@"'"];
	NSTimeZone *timeZoneUTC = [NSTimeZone timeZoneWithName:@"UTC"];
	NSString* dateString=[dateOfData description];
	NSCalendarDate* tempCalDate=[NSCalendarDate dateWithString:dateString];
	[tempCalDate setTimeZone:timeZoneUTC];
	NSString* utcDateString=[tempCalDate descriptionWithCalendarFormat:@"%Y-%m-%d %H:%M:%S"];
	data=[data stringByAppendingString:utcDateString];
	data=[data stringByAppendingString:@"'"];
    columns=[columns stringByAppendingString:@"Time"];

//Daily Rain
/*
    float value=[[[myData objectForKey:@"rain_daily"]  objectForKey:@"value"] floatValue];
    BOOL valid=[[[myData objectForKey:@"rain_daily"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", DailyRain_mm"];
    }
*/
//Monthly Rain
    float value=[[[myData objectForKey:@"rain_monthly"]  objectForKey:@"value"] floatValue];
    BOOL valid=[[[myData objectForKey:@"rain_monthly"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", MonthlyRain_mm"];
    }

//Yearly Rain
    value=[[[myData objectForKey:@"rain_yearly"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"rain_yearly"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", YearlyRain_mm"];
    }


//Daily ET
    value=[[[myData objectForKey:@"et_daily"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"et_daily"]  objectForKey:@"valid"] floatValue];

	if (valid)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", DailyET"];
    }
    

//Monthly ET
    value=[[[myData objectForKey:@"et_monthly"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"et_monthly"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", MonthlyET"];
    }
//Yearly ET
    
    value=[[[myData objectForKey:@"et_yearly"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"et_yearly"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", YearlyET"];
    }


//T1
    value=[[[myData objectForKey:@"aux_temperature1"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_temperature1"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", T1_C"];
    }

//T2
    value=[[[myData objectForKey:@"aux_temperature2"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_temperature2"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", T2_C"];
    }

//T3
    value=[[[myData objectForKey:@"aux_temperature3"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_temperature3"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", T3_C"];
    }
//T4
    value=[[[myData objectForKey:@"aux_temperature4"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_temperature4"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", T4_C"];
    }
//T5
    value=[[[myData objectForKey:@"aux_temperature5"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_temperature5"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", T5_C"];
    }
//T6
    value=[[[myData objectForKey:@"aux_temperature6"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_temperature6"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", T6_C"];
    }
//T7
    value=[[[myData objectForKey:@"aux_temperature7"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_temperature7"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", T7_C"];
    }
//T8
    value=[[[myData objectForKey:@"aux_temperature8"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_temperature8"]  objectForKey:@"valid"] floatValue];

	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", T8_C"];
    }


//H1
    value=[[[myData objectForKey:@"aux_rh1"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_rh1"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", H1"];
    }
//H2
    value=[[[myData objectForKey:@"aux_rh2"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_rh2"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", H2"];
    }
//H3
    value=[[[myData objectForKey:@"aux_rh3"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_rh3"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.0f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", H3"];
    }
//H4
    value=[[[myData objectForKey:@"aux_rh4"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_rh4"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.0f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", H4"];
    }
//H5
    value=[[[myData objectForKey:@"aux_rh5"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_rh5"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.0f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", H5"];
    }
//H6
    value=[[[myData objectForKey:@"aux_rh6"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_rh6"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.0f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", H6"];
    }
//H7
    value=[[[myData objectForKey:@"aux_rh7"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_rh7"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.0f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", H7"];
    }
//H8
    value=[[[myData objectForKey:@"aux_rh8"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"aux_rh8"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.0f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", H8"];
    }

//Leaf Temp 1
    value=[[[myData objectForKey:@"leaf_temperature1"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"leaf_temperature1"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", LeafTemp1_C"];
    }
//Leaf Temp 2
    value=[[[myData objectForKey:@"leaf_temperature2"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"leaf_temperature2"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", LeafTemp2_C"];
    }
//Leaf Temp 3
    value=[[[myData objectForKey:@"leaf_temperature3"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"leaf_temperature3"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", LeafTemp3_C"];
    }
//Leaf Temp 4
    value=[[[myData objectForKey:@"leaf_temperature4"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"leaf_temperature4"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", LeafTemp4_C"];
    }

//Soil Temp 1
    value=[[[myData objectForKey:@"soil_temperature1"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"soil_temperature1"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", SoilTemp1_C"];
    }

//Soil Temp 1
    value=[[[myData objectForKey:@"soil_temperature2"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"soil_temperature2"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", SoilTemp2_C"];
    }
//Soil Temp 3
    value=[[[myData objectForKey:@"soil_temperature3"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"soil_temperature3"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", SoilTemp3_C"];
    }
//Soil Temp 4
    value=[[[myData objectForKey:@"soil_temperature4"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"soil_temperature4"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", SoilTemp4_C"];
    }

//Leaf Wetness 1
    value=[[[myData objectForKey:@"leaf_wetness1"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"leaf_wetness1"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", LeafWet1"];
    }

//Leaf Wetness 2
    value=[[[myData objectForKey:@"leaf_wetness2"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"leaf_wetness2"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", LeafWet2"];
    }
//Leaf Wetness 3
    value=[[[myData objectForKey:@"leaf_wetness3"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"leaf_wetness3"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", LeafWet3"];
    }
//Leaf Wetness 4
    value=[[[myData objectForKey:@"leaf_wetness4"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"leaf_wetness4"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", LeafWet4"];
    }

//Soil Moisture 1
    value=[[[myData objectForKey:@"soil_moisture1"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"soil_moisture1"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", SoilMoist1"];
    }

//Soil Moisture 2
    value=[[[myData objectForKey:@"soil_moisture2"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"soil_moisture2"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", SoilMoist2"];
    }
//Soil Moisture 3
    value=[[[myData objectForKey:@"soil_moisture3"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"soil_moisture3"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", SoilMoist3"];
    }
//Soil Moisture 4
    value=[[[myData objectForKey:@"soil_moisture4"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"soil_moisture4"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", SoilMoist4"];
    }



	data=[data stringByAppendingString:@");"];
	columns=[columns stringByAppendingString:@") "];

    send_string=[send_string stringByAppendingString:columns];
    send_string=[send_string stringByAppendingString:data];
    if (data_count>0)   //don't send anything if there's no data
    {
        int error=mysql_query(con, [send_string UTF8String]);

        if (error == MYSQL_OK)
        {
            internal_sql_error = MYSQL_OK;
            bytes_sent+=[send_string length];

        }
        else
        {
            unsigned int error=mysql_errno(con);
            NSString* logMessage=[NSString stringWithFormat:@"%s\n",mysql_error(con)];
            if (error==1045)
            {
                current_state=updater_pwd;
            }
            else
            if (error==1049)
            {
                current_state=updater_unknown_db;
            }
            else
            if (error==2003)
            {
                current_state=updater_cant_connect;
            }
            else
            {
                current_state=updater_sql_error;
            }
            NSLog(logMessage);
            [self sendLogMessage:logMessage];

            internal_sql_error = MYSQL_SQL;
            current_state=updater_unknown_db;
        }
    }
    else
    {
        internal_sql_error = MYSQL_OK;
    }

return 0;
}





-(int) sendWindData
{
NSString* send_string=@"INSERT INTO WindData ";
NSString* data=@"VALUES (";
NSString* columns=@"(";

NSString* temp;
NSDate* dateOfData=[NSDate date];
//    NSDictionary* myData=[myAPIManager getCurrentWeatherData];  //Get all available data

//Add date and time
	data=[data stringByAppendingString:@"'"];
	NSTimeZone *timeZoneUTC = [NSTimeZone timeZoneWithName:@"UTC"];
	NSString* dateString=[dateOfData description];
	NSCalendarDate* tempCalDate=[NSCalendarDate dateWithString:dateString];
	[tempCalDate setTimeZone:timeZoneUTC];
	NSString* utcDateString=[tempCalDate descriptionWithCalendarFormat:@"%Y-%m-%d %H:%M:%S"];
	data=[data stringByAppendingString:utcDateString];
	data=[data stringByAppendingString:@"'"];

    columns=[columns stringByAppendingString:@"Time"];

//Windspeed
    int channel=[myAPIManager getFirstChannelOfType: windspeed_kmh_instantaneous];
    float value=[myAPIManager getChannelValue:channel];
    BOOL valid=[myAPIManager getChannelStatus:channel];
	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindSpeed_kmh"];
    }
//Winddir
    channel=[myAPIManager getFirstChannelOfType: winddirection_degrees_instantaneous];
    value=[myAPIManager getChannelValue:channel];
    valid=[myAPIManager getChannelStatus:channel];
	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindDir"];
    }
//Av Windspeed
    channel=[myAPIManager getFirstChannelOfType: windspeed_kmh_average_10mins];
    value=[myAPIManager getChannelValue:channel];
    valid=[myAPIManager getChannelStatus:channel];
	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindSpeedAv_kmh"];
    }
//WinddirAv
    channel=[myAPIManager getFirstChannelOfType: winddirection_degrees_average];
    value=[myAPIManager getChannelValue:channel];
    valid=[myAPIManager getChannelStatus:channel];
	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindDirAv"];
    }
//Gust Windspeed
    channel=[myAPIManager getFirstChannelOfType: windgust_kmh_10mins];
    value=[myAPIManager getChannelValue:channel];
    valid=[myAPIManager getChannelStatus:channel];
	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindSpeedGust_kmh"];
    }
//WinddirGust10m
    channel=[myAPIManager getFirstChannelOfType: windgust_dir_10mins];
    value=[myAPIManager getChannelValue:channel];
    valid=[myAPIManager getChannelStatus:channel];
	if (valid==YES)
    {
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindDirGust"];
    }

	data=[data stringByAppendingString:@");"];
	columns=[columns stringByAppendingString:@") "];

    send_string=[send_string stringByAppendingString:columns];
    send_string=[send_string stringByAppendingString:data];

    sent_packets++;
    int error=mysql_query(con, [send_string UTF8String]);

    if (error == MYSQL_OK)
    {
        internal_sql_error = MYSQL_OK;
        bytes_sent+=[send_string length];
        current_state=updater_ok;

        ack_packets++;
    }
    else
    {
        unsigned int error=mysql_errno(con);
        NSString* logMessage=[NSString stringWithFormat:@"%s\n",mysql_error(con)];
        if (error==1045)
        {
            current_state=updater_pwd;
        }
        else
        if (error==1049)
        {
            current_state=updater_unknown_db;
        }
        else
        if (error==2003)
        {
            current_state=updater_cant_connect;
        }
        else
        {
            current_state=updater_sql_error;
        }
        NSLog(logMessage);
        [self sendLogMessage:logMessage];

        current_state=updater_unknown_db;
        internal_sql_error = MYSQL_SQL;
    }


return 0;
}


//We have a separate upload for historic data as we don't currently store instantaneous windspeed or gust direction. 
-(int) sendWindDataHistoric:(NSDictionary*) myData date:(NSDate*) dateOfData
{
NSString* send_string=@"INSERT INTO WindData ";
NSString* data=@"VALUES (";
NSString* columns=@"(";

NSString* temp;
//    NSDictionary* myData=[myAPIManager getCurrentWeatherData];  //Get all available data

//Add date and time
	data=[data stringByAppendingString:@"'"];
	NSTimeZone *timeZoneUTC = [NSTimeZone timeZoneWithName:@"UTC"];
	NSString* dateString=[dateOfData description];
	NSCalendarDate* tempCalDate=[NSCalendarDate dateWithString:dateString];
	[tempCalDate setTimeZone:timeZoneUTC];
	NSString* utcDateString=[tempCalDate descriptionWithCalendarFormat:@"%Y-%m-%d %H:%M:%S"];
	data=[data stringByAppendingString:utcDateString];
	data=[data stringByAppendingString:@"'"];

    columns=[columns stringByAppendingString:@"Time"];
    int data_count=0;
//Winddir
    float value=[[[myData objectForKey:@"winddirection_average"]  objectForKey:@"value"] floatValue];
    BOOL valid=[[[myData objectForKey:@"winddirection_average"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindDir"];
    }
//Av Windspeed
    value=[[[myData objectForKey:@"windspeed_average"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"windspeed_average"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindSpeedAv_kmh"];
//Use also as instantaneous (we don't currently store this)
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindSpeed_kmh"];

    }
//WinddirAv
    value=[[[myData objectForKey:@"winddirection_average"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"winddirection_average"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindDirAv"];
//Use also as gust dir (we don't currently store this)
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindDirGust"];

    }
//Gust Windspeed
    value=[[[myData objectForKey:@"windspeed_gust"]  objectForKey:@"value"] floatValue];
    valid=[[[myData objectForKey:@"windspeed_gust"]  objectForKey:@"valid"] floatValue];
	if (valid==YES)
    {
        data_count++;
        temp=[NSString stringWithFormat:@",'%.1f'",value];
        data=[data stringByAppendingString:temp];
        columns=[columns stringByAppendingString:@", WindSpeedGust_kmh"];
    }

	data=[data stringByAppendingString:@");"];
	columns=[columns stringByAppendingString:@") "];

    send_string=[send_string stringByAppendingString:columns];
    send_string=[send_string stringByAppendingString:data];
    if (data_count>0)   //anything to send?
    {
        int error=mysql_query(con, [send_string UTF8String]);

        if (error == MYSQL_OK)
        {
            internal_sql_error = MYSQL_OK;
            bytes_sent+=[send_string length];

        }
        else
        {
             current_state=updater_unknown_db;
           internal_sql_error = MYSQL_SQL;
        }
    }
    else
    {
            internal_sql_error = MYSQL_OK;
    }
return 0;
}


-(int) createDataTable
{
//Time is in UTC
NSString* create=@"CREATE TABLE IF NOT EXISTS WeatherData (Time timestamp, Temperature_C float, Dewpoint_C float, Humidity float, Pressure_mB float, ";
create=[create stringByAppendingString:@"RainRate_mm float, RawRain_mm float, "];
create=[create stringByAppendingString:@"Solar_wm2 float, UV_index float, TemperatureInt_C float, HumidityInt float, Conditions VARCHAR(200));"];
    int error=mysql_query(con, [create UTF8String]);

    if (error == MYSQL_OK)
    {
        internal_sql_error=MYSQL_OK;
    }
    else
    {
        internal_sql_error = MYSQL_SQL;
    }

//            SqlCommand myCommand = new SqlCommand(str, myConn);
return 0;
}


-(int) createExtraTable
{
//Time is in UTC
NSString* create=@"CREATE TABLE IF NOT EXISTS ExtraData (Time timestamp, MonthlyRain_mm float, YearyRain_mm float, DailyET float, MonthlyET float, YearlyET float, ";
create=[create stringByAppendingString:@"T1_C float, T2_C float, T3_C float, T4_C float, T5_C float, T6_C float, T7_C float, T8_C float, "];
create=[create stringByAppendingString:@"H1 float, H2 float, H3 float, H4 float, H5 float, H6 float, H7 float, H8 float, "];
create=[create stringByAppendingString:@"LeafTemp1_C float, LeafTemp2_C float, LeafTemp3_C float, LeafTemp4_C float, SoilTemp1_C float, SoilTemp2_C float, SoilTemp3_C float, SoilTemp4_C float, "];
create=[create stringByAppendingString:@"LeafWet1 float, LeafWet2 float, LeafWet3 float, LeafWet4 float, SoilMoist1 float, SoilMoist2 float, SoilMoist3 float, SoilMoist4 float);"];
    int error=mysql_query(con, [create UTF8String]);

    if (error == MYSQL_OK)
    {
        internal_sql_error=MYSQL_OK;
    }
    else
    {
        internal_sql_error = MYSQL_SQL;
    }

return 0;
}


-(int) createWindTable
{
//Time is in UTC
NSString* create=@"CREATE TABLE IF NOT EXISTS WindData (Time timestamp, ";
create=[create stringByAppendingString:@"Windspeed_kmh float, Winddir float, WindspeedAv_kmh float, WinddirAv float, WindspeedGust_kmh float, "];
create=[create stringByAppendingString:@"WindDirGust float);"];

    int error=mysql_query(con, [create UTF8String]);

    if (error == MYSQL_OK)
    {
        internal_sql_error=MYSQL_OK;
    }
    else
    {
        internal_sql_error = MYSQL_SQL;
    }

return 0;
}


//This table contains vital information for the future when we need to do additions for example
//It tells us what the version of the data is.
-(int) createPrivateTable
{
//Time is in UTC
NSString* create=@"CREATE TABLE IF NOT EXISTS _WeatherCatPrivate (dataVersion int);";
NSString* version_string=@"INSERT INTO _WeatherCatPrivate VALUES (1); "; //we are currently data version 1
    int error=mysql_query(con, [create UTF8String]);

    if (error == MYSQL_OK)
    {
        error=mysql_query(con, [version_string UTF8String]);
        if (error==MYSQL_OK)
        {
            internal_sql_error=MYSQL_OK;
        }
        else
        {
            internal_sql_error = MYSQL_SQL;
        }
    }
    else
    {
        internal_sql_error = MYSQL_SQL;
    }

return 0;
}



-(bool) sendLogMessage:(NSString*) message
{
		NSString* send_string=[NSString stringWithFormat:@"SQLDriver: %@", message];
        
        [[myController getWeatherCat] logString:send_string];
		//[myController performSelectorOnMainThread:@selector(do_log_entry:) withObject:send_string waitUntilDone:NO];
	return YES;

}


-(void) resetTable
{
    checked_tables=NO;
}

-(void) resetTimings
{
    AverageTime=0;
    CumulativeFlightTime=0;
}


//Get/Set the last upload time - you'd think we can do this from the DB but there are too many variations to contend with
//so we store it in a file.
#define WC_TEMP_PATH @"~/Library/Application Support"

-(NSDate*) getLastUploadDate
{
 NSString* core_path;
 NSString* current_path;
 NSMutableDictionary* data;
 BOOL is_directory;
    
    core_path= [[NSString localizedStringWithFormat:WC_TEMP_PATH] stringByExpandingTildeInPath];
    core_path=[core_path stringByAppendingString:@"/WeatherCatTemp/"];
    
    //Check if the WeatherCatTemp directory exists, if it doesn't, create it.
    BOOL create_result=[[NSFileManager defaultManager]fileExistsAtPath:core_path isDirectory:&is_directory];
    if (create_result==NO) //need to make the directory and allowed to
    {
	    NSError* myError=[NSError errorWithDomain:@"WeatherCat" code:0 userInfo:0];
	    create_result=[[NSFileManager defaultManager]createDirectoryAtPath:core_path withIntermediateDirectories:NO attributes:nil error:&myError];
	    
	    if (create_result==NO)
	    {
		    NSString* logStr=[NSString localizedStringWithFormat:NSLocalizedString (@"***Error*** Couldn't make directory for temporary files. Path was: %@", @""),core_path];
		    [self sendLogMessage:logStr];
		    //return;
	    }
    }

    current_path=[core_path stringByAppendingPathComponent:@"WeatherCatLastSQLUpload.plist"];
    data=[NSMutableDictionary dictionaryWithContentsOfFile:current_path];
    //does the file exist? If not, return now
    if (data)
    {
        NSDate* returnDate=[NSDate dateWithTimeIntervalSinceReferenceDate:[[data objectForKey:@"lastUploadTime"] floatValue]];
        return returnDate;
    }
    else
    {
        return [NSDate date];	//return now
    }
}

-(void) setLastUploadDate
{
    [self setLastUploadDateWithDate:[NSDate date]];
}


-(void) setLastUploadDateWithDate:(NSDate*) date
{
 NSString* core_path;
 NSString* current_path;
 NSMutableDictionary* data;
 BOOL is_directory;

    
    data=[NSMutableDictionary dictionaryWithCapacity:1];
    [data setObject:[NSNumber numberWithFloat:[date timeIntervalSinceReferenceDate ]] forKey:@"lastUploadTime"];
    
    core_path= [[NSString localizedStringWithFormat:WC_TEMP_PATH] stringByExpandingTildeInPath];
    core_path=[core_path stringByAppendingString:@"/WeatherCatTemp/"];

    BOOL create_result=[[NSFileManager defaultManager]fileExistsAtPath:core_path isDirectory:&is_directory];
    if (create_result==NO) //need to make the directory and allowed to
    {
		NSError* myError=[NSError errorWithDomain:@"WeatherCat" code:0 userInfo:0];
		create_result=[[NSFileManager defaultManager]createDirectoryAtPath:core_path withIntermediateDirectories:NO attributes:nil error:&myError];
	    
	    if (create_result==NO)
	    {
		NSString* logStr=[NSString localizedStringWithFormat:NSLocalizedString (@"***Error*** Couldn't make directory for temporary files. Path was: %@", @""),core_path];
		[self sendLogMessage:logStr];
	    }
    }

    current_path=[core_path stringByAppendingPathComponent:@"WeatherCatLastSQLUpload.plist"];
    [data writeToFile:current_path atomically:YES]; //write to temp, then rename to path
}

-(void) quit
{
    thread_stop=YES;
    usleep(1000000);    //give any in-porgress updates time to stop
}
@end
